﻿//ILN(enabled = false)
using ILNumerics;
using static ILNumerics.ILMath;
using static ILNumerics.Globals;
using System.Diagnostics;

Array<double> A = counter<double>(1.0, 1.0, 400, 400);
Array<double> A_ILN = A.C;
Array<double> A_Parfor = A.C;
Array<double> A_Acc = A.C;

(long iln, long parf, long acc) run() {

    var sw = Stopwatch.StartNew();
    for (int i = 0; i < A.S[1]; i++) {
        A_ILN[full, r(0, i)] = abs(sum(sin(A[full, r(0, i)]), dim: 1));
    }

    // Parallel loop, out-of order
    var sw_p = Stopwatch.StartNew();
    Parallel.For(0, A.S[1], i => {
        A_Parfor[full, r(0, i)] = abs(sum(sin(A[full, r(0, i)]), dim: 1));
    });
    sw_p.Stop();

    // Accelerator (in-order) 
    //ILN(enabled = true)
    Stopwatch sw_acc = Stopwatch.StartNew();
    for (int i = 0; i < A.S[1]; i++) {
        A_Acc[full, r(0, i)] = abs(sum(sin(A[full, r(0, i)]), dim: 1));
    }
    A_Acc.Finish();
    sw_acc.Stop();

    return (sw.ElapsedMilliseconds, sw_p.ElapsedMilliseconds, sw_acc.ElapsedMilliseconds);
}

// run 20 repetitions, display times & check result
for (int i = 0; i < 20; i++) {
    var times = run();
    Console.Write($"ILN: {times.iln} ");
    Console.Write($"\tParFor: {times.parf} "); check((double)maxall(abs(A_Parfor - A_ILN)));
    Console.Write($"\tACC: {times.acc} "); check((double)maxall(abs(A_Acc - A_ILN)));
    Console.WriteLine();
}

void check(double err) {
    if (err < eps) {
        Console.Write("(correct: ");
        Console.ForegroundColor = ConsoleColor.Green;
        Console.Write("True");
        Console.ResetColor();
        Console.Write(")");
    } else {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.Write($"(err: {err:f6})");
        Console.ResetColor();
    }
}